library thai;

bool isFirstAlphaBetTH(String c) {
  var i = c.runes.first;
  return 'ก'.runes.first <= i && i <= '๙'.runes.first;
}

String getFirstTHChar(String s) {
  if (s == null || s.length == 0) {
    return null;
  }
  return String.fromCharCode(s.runes.firstWhere((c) {
    return 'ก'.runes.first <= c && c <= 'ฮ'.runes.first;
  }));
}

int compareStringDictionary(String first, String second) {
  String replaceToneMarks(String s) {
    if (s == null || s.length == 0) {
      return null;
    }
    RegExp regExp = RegExp(r'(็|่|้|๊|๋|์)');
    return s.replaceAll(regExp, '');
  }

  if ((first != null && first.length > 0) && (second != null && second.length > 0)) {
    if (!isFirstAlphaBetTH(first) || !isFirstAlphaBetTH(second)) {
      return first.compareTo(second);
    }
    var rmFirstToneMark = replaceToneMarks(first);
    var rmSecondToneMark = replaceToneMarks(second);

    var firstTHChar = getFirstTHChar(rmFirstToneMark);
    var secondTHChar = getFirstTHChar(rmSecondToneMark);
    if (firstTHChar != secondTHChar) {
      return firstTHChar.compareTo(secondTHChar);
    }

    if (rmFirstToneMark != rmSecondToneMark) {
      return rmFirstToneMark.compareTo(rmSecondToneMark);
    }
  }

  return first.compareTo(second);
}
